﻿#---------------------------------------------------------------------------------------------------------------------------*
#  Purpose        : Script for Information gathering of Azure PostgreSQL Single Server
#  Schedule       : Ad-Hoc / On-Demand
#  Date           : 19-July-2023
#  Author         : Rackimuthu Kandaswamy , Sireesha , ArunKumar , Saby , Lekshmy MK, Madan Agrawal
#  Version        : 2.1
#   
#  INPUT          : NONE
#  VARIABLE       : NONE
#  PARENT         : NONE
#  CHILD          : NONE
#---------------------------------------------------------------------------------------------------------------------------*
#---------------------------------------------------------------------------------------------------------------------------*
#
#  IMPORTANT NOTE : The script has to be run on Non-Mission-Critical systems ONLY and not on any production server...
#
#---------------------------------------------------------------------------------------------------------------------------*
#---------------------------------------------------------------------------------------------------------------------------*
# Usage:
# Powershell.exe -File .\CMF-PostgreSQL-CLI-Windows.ps1
#
<#
    Change Log
    ----------
•	Customer consent to install Azure CLI
•	Excluded Flexi server Azure CLI commands.
•	Excluded Azure CLI commands Json file generation for single servers except Server list.
•	Incorporated Vcore column in INPUT CSV  Sheet  CMF_PostgreSQL_Server_Input_file.xlsx file (Server_List Worksheet)
•	Incorporated servername in DB_List, Config_Details,AD Admin and Firewall-rule List  CLI commands output in output.xlsx
•	No column Filter applied when exporting AZURE CLI Commands output to output.xlsx file
•	Included az postgres server replica list CLI
•	Script now can be run directly without wrapper.
•	Output files creation verification added at end of script.
•	-NoTypeInformation to suppress the first line from CSVs.        
•	Skip execution for servers where server state is not "Ready" to suppress error message and to gain performace.
•	Progress bar addition with server in progress
•	Included Multi-subscription support        
#>
Set-ExecutionPolicy Bypass -Scope currentuser
CLS


#---------------------------------------------------------PROGRAM BEGINS HERE----------------------------------------------------------

write-host "                                                                            " -BackgroundColor DarkMagenta
Write-Host "     Welcome to CMF - PostgreSQL_Info_Gathering_Automation                  " -ForegroundColor white -BackgroundColor DarkMagenta
write-host "                     (OSS DB Migration Factory)                             " -BackgroundColor DarkMagenta
write-host "                                                                            " -BackgroundColor DarkMagenta
Write-Host " "

$folder = $PSScriptRoot
$postgrespath = where.exe psql
$today_date=Get-Date -Format "MM_dd_yyyy_HH_mm"
 Write-Host "=============================================================================================================="  
 Start-Transcript -path  $folder\Logs\CMF-PostgreSQL_Info_Gathering_Automation_Transcript_$today_date.txt  -Append
 Write-Host "=============================================================================================================="  
function exitCode
{
    Write-Host "-Ending Execution"
    Stop-Transcript
    exit
}

function createFolder([string]$newFolder) 
{
   if(Test-Path $newFolder)
    {
        Write-Host "-Folder'$newFolder' Exist..."
    }
    else
    {
        $cfolder=New-Item $newFolder -ItemType Directory
        Write-Host "-$newFolder folder created..."
    }
}

if ($postgrespath -like "*\psql*") {
	Write-Host "PostgreSQL exists. Continuing execution..." -ForegroundColor Green
}
else {
	Write-Host "PostgreSQL is not installed or its path is not in the environment variables. Please install PostgreSQL, add its path to the environment variables, and rerun the script." -BackgroundColor Red
	exit
}

createFolder $folder\Downloads\
createFolder $folder\Logs\
createFolder $folder\Output\
createFolder $folder\Output\Single\

# Read the input config CSV and validate
$inputfile = $PSScriptRoot+"\Azure_Subscription.csv"
Write-Host "Input file is $inputfile " -ForegroundColor Green
Write-Host "=============================================================================================================="  


if (-not(Test-Path -Path $inputfile -PathType Leaf)) {
    try {    
         Write-Host "=============================================================================================================="   
         Write-Host "Unable to read the input file [$inputfile]. Check file & its permission...  "  -BackgroundColor Red
         Write-Host "=============================================================================================================="  
         Write-Host "Please see the error below Execution has been stopped          "  
         throw $_.Exception.Message                      
    }
    catch {
         throw $_.Exception.Message
    }
 }
else
{
     try {
         $ConfigList = Import-CSV -Path $inputfile  
         }

         catch {
         Write-Host "=============================================================================================================="  
         Write-Host "The file [$inputfile] does not have the woksheet named Azure_Subscription "  -BackgroundColor Red 
         Write-Host "=============================================================================================================="   
         Write-Host "Please see the error below & Azure PostgreSQL Info-Gathering has been stopped          "  
         throw $_.Exception.Message
         exitcode
        }

}    
 
 $ColumnList=$ConfigList | Get-Member -MemberType NoteProperty | %{"$($_.Name)"}
     if (($ColumnList.Contains("Tenant")) -and
        ($ColumnList.Contains("Subscription_ID"))){

        Write-Host "CSV validation is done successfully " 
        }
     else {Write-Host "There are mismatches in the CSV column . Kindly check and retrigger the automation "  -BackgroundColor Red
           exitCode}


Unblock-File $folder/Validation_Scripts/Check_PowerShell_Version.ps1
Unblock-File $folder/Validation_Scripts/azurecli.ps1
      
# Check PowerShell version
$Validation=@()
$Outfiledata=@()
$global:alldata =@()
$global:row=@{}
$Validation+=& "$folder/Validation_Scripts/Check_PowerShell_Version.ps1"
$Validation+=& "$folder/Validation_Scripts/azurecli.ps1"

 Write-Host "=============================================================================================================="  
 Write-Host "Below are the Validation Results"  -ForegroundColor Green  
 Write-Host "=============================================================================================================="  
Write-Host ($Validation | select Validation_Type,Status,Comments | Format-Table | Out-String)

If($Validation.Status.Contains("FAILED"))
{
 Write-Host "There are errors during validation . Terminating the execution ."  -ForegroundColor Red
 exitcode  
}


#Function to convert json value to hashtable

    function ConvertTo-Hashtable {
    [CmdletBinding()]
    [OutputType('hashtable')]
    param (
        [Parameter(ValueFromPipeline)]
        $InputObject
          )

        process {
        if ($null -eq $InputObject) 
        {
            return $null
         }
        
        if ($InputObject -is [System.Collections.IEnumerable] -and $InputObject -isnot [string])
        {
            $collection = @(
                foreach ($object in $InputObject) {
                    ConvertTo-Hashtable -InputObject $object
                }
                )

         Write-Output -NoEnumerate $collection
        }
        elseif ($InputObject -is [psobject]) 
        {
            $hash = @{}
            
            foreach ($property in $InputObject.PSObject.Properties) 
            {
                $hash[$property.Name] = ConvertTo-Hashtable -InputObject $property.Value
            }
            
             $hash
        } 
        else 
        {
            $InputObject
        }
        }
    }

    #Functions to fetch the json paths
    function CMF-Read-Hashtable ([Hashtable]$InputHashTable){
	foreach ($h in $InputHashTable.GetEnumerator() ){
		if ($h.Value -is [Array]){
			CMF-Read-Nested-Array $h.Name $h.Value
		}elseif ($h.Value -is [Hashtable]){
			CMF-Read-Nested-Hashtable $h.Name $h.Value
		}else{
                       $key = $h.Name
                       if ($global:row.ContainsKey($key)) {
                         $global:row[$key] = $h.Value
                       }
                       else {   
			 $global:row.Add($key, $h.Value)
                       }
		}
	}
	$global:alldata += $global:row
	$global:row =@{}
    }

    function CMF-Read-Nested-Hashtable ([String]$prefix,[Hashtable]$InputNestedHashTable){
	foreach ($h in $InputNestedHashTable.GetEnumerator()){
		if ($h.Value -is [Array]){
			CMF-Read-Nested-Array $h.Name $h.Value
		}elseif ($h.Value -is [Hashtable]){
            $newPrefix=-join($prefix,".",$h.Name)
			CMF-Read-Nested-Hashtable $newPrefix $h.Value
		}else{
                       $key = -join @($prefix, ".", $h.Name)
                       if ($global:row.ContainsKey($key)) {
                         $global:row[$key] = $h.Value
                       }
                       else {   
			 $global:row.Add($key, $h.Value)
                       }
		}
	    }
    }

    function CMF-Read-Array ([Array]$InputArray){
    foreach ($a in $InputArray){
		if($a -is [Hashtable]){
			CMF-Read-Hashtable $a
		}elseif ($a.Value -is [Array]){
		    CMF-Read-Nested-Array $a.Name $a.Value 
		}elseif ($a -is [Array]){
			CMF-Process-Nested-Array $null $a	
		}elseif($a.Value -is [Hashtable]){
			CMF-Read-Nested-Hashtable $a.Name $a.Value
		}else{
			Write-Host "Process-Array :$($a.Value.pstypenames)"
		}
	    }
    }

    function CMF-Read-Nested-Array ([String]$prefix,[Array]$InputArray){
    foreach ($a in $InputArray){
		if($a -is [Hashtable]){
		    CMF-Read-Nested-Hashtable $prefix $a
		}elseif ($a.Value -is [Array]){
			CMF-Read-Nested-Array $prefix $a.Value 
		}elseif ($a -is [Array]){
			CMF-Read-Nested-Array $prefix $a
		}elseif($a.Value -is [Hashtable]){
			CMF-Read-Nested-Hashtable $a.Name $a.Value
		}else{
			$global:row.Add(-join($prefix,".",$h.Name), $h.Value)
		}
	    }
    }



$tenant=$ConfigList.Tenant|Get-Unique

    if ([string]::IsNullOrWhitespace($tenant)){
Write-Host "Tenant listed Azure_Subscription worksheet is not valid. Kindly check and retrigger the automation  "  -BackgroundColor Red 
exitCode
}
elseif($tenant.count -gt 1)
{
Write-Host "Azure_Subscription.csv has multiple Tenant. This script doesn't support multiple Tenant but you can add multiple Subscription. `nKindly check and retrigger the automation for each Tenant separately."  -BackgroundColor Red 
exitCode

}
else
{
#AZ login to corresponsing Tenant and subscription
#$loginoutput=az login --tenant $tenant --only-show-errors
$loginoutput=az login --use-device-code --tenant $tenant --only-show-errors
}

if (!$loginoutput) 
{
    Write-Error "Error connecting to Tenant: $tenant"
    exitcode
}
$Subscriptions= $ConfigList.Subscription_ID|Sort-Object -Unique

      foreach ($Subscription in $Subscriptions)
  {
    $Serverdata=@()
    $db_data=@()
    $DBList=@()
    $ServerConf=@()
    $confdata=@()
    $ADAdmin=@()
    $Admin=@()
    $Firewall=@()
    $Replica=@()
    $CMKey=@()
    $MSDef=@()
    $SerEnd=@()
    $PvtEnd=@()
    $dbdata=@()


if ([string]::IsNullOrWhitespace($Subscription)){
Write-Host "$Subscription is not valid in the Azure_Subscription worksheet. Kindly check and retrigger the automation  "  -BackgroundColor Red 
continue
} 


    Write-Host "==============================================================================================================" 
    Write-Host "Processing Subscription: $Subscription" -ForegroundColor Green
    Write-Host "==============================================================================================================" 
   
    #AZ Connect to provided subscription
   
         $loginoutput=az account set --subscription $Subscription 2>&1
           $stat = $LASTEXITCODE


If( $stat -ne 0)
    {
       # seems to have failed
       #$errorMessage = $Error[0].Exception.Message
         Write-Host "Failed to execute [az account set --subscription $Subscription]" -ForegroundColor Red
         Write-Host "The file [$inputfile] does not have valid Subscription.`n"  -ForegroundColor Red
        continue
    }
     
    #Server list fetch for Single Server
    $Single_list_Out = az postgres server list | Out-File $folder\Output\Single\Single_Server_List.json -Append
    
    $ser_details = az postgres server list|ConvertFrom-Json
      
    $ser_list = az postgres server list |ConvertFrom-Json | ConvertTo-HashTable
    if($ser_list -is [Hashtable])
    {
        CMF-Read-Hashtable $ser_list
    }else
    {
	    CMF-Read-Array $ser_list
    }
    
    $Serverdata+=$global:alldata
    $global:alldata=@()
     $count=  $ser_details.count
    $k=1
     #Write-Host "=============================================================================================================="  
    if ($count -eq 0)
    {
    Write-Host "There is no PostgreSQL single server on Subscription [$Subscription] for Tenant [$tenant]" -ForegroundColor Red
    Write-host "Please check Subscription on input file: Azure_Subscription.csv" -ForegroundColor Red
    #Write-Host "=============================================================================================================="  
   # exitCode
    }
  foreach ($i in $ser_details)
  {
           
      #Updating input file with server details
      $ServerName= $i.name
      $Single_User_Name = $i.administratorLogin
      $ser_rg = $i.resourceGroup  
      $domain_name = $i.fullyQualifiedDomainName
      $dom = $domain_name.Split(".")
      $ser_dom = $dom[0]
      $Logon_User=$Single_User_Name + "@" + $ser_dom
      $vcore=$i.sku.capacity
      $userVisibleState=$i.userVisibleState
       $per_complete=[math]::round($k/$count*100)
      Write-Progress -Activity "Server:$ServerName is in Progress..." -Status "$per_complete% Complete:" -PercentComplete $per_complete
      $k++
      $Outfiledata+=New-Object psobject -Property @{Host_Name=$domain_name;Resource_Group=$ser_rg;Port="5432";VCore=$vcore;Auth_Type="PostgreSQL";User_ID=$Logon_User;Password="";DB_Name="postgres";Tenant=$tenant;Subscription_ID=$Subscription;Approval_Status="No";SSL_Mode="prefer"}
  if ($userVisibleState -match "Ready")
  {           
    try
    {    
    $dbdata = az postgres db list -g $ser_rg -s $ser_dom | ConvertFrom-Json | ForEach-Object { $_ | Add-Member -MemberType NoteProperty -Name 'Servername' -Value $ServerName -PassThru} | ConvertTo-Json
    $db_data= $dbdata  |ConvertFrom-Json |  ConvertTo-HashTable
    if($db_data -is [Hashtable])
    {
        CMF-Read-Hashtable $db_data
    }else
    {
	    CMF-Read-Array $db_data
    }
    $DBList+=$global:alldata
    $global:alldata=@()

    #Server Configuration
    
    #$conf_list = az postgres server configuration list -g $ser_rg --server-name $ser_dom | Out-File $folder\Output\Single\$ServerName\${ServerName}_Single_Server_Configuration.json
    $confdata = az postgres server configuration list -g $ser_rg -s $ser_dom | ConvertFrom-Json| ForEach-Object { $_ | Add-Member -MemberType NoteProperty -Name 'Servername' -Value $ServerName -PassThru} | ConvertTo-Json
    $confdata = $confdata|ConvertFrom-Json | ConvertTo-HashTable
    if($confdata -is [Hashtable])
    {
        CMF-Read-Hashtable $confdata
    }else
    {
	    CMF-Read-Array $confdata
    }
    
    $ServerConf+=$global:alldata
    $global:alldata=@()

    #AD Admin
    
    #$ad_admin = az postgres server ad-admin list -g $ser_rg -s $ser_dom | Out-File $folder\Output\Single\$ServerName\${ServerName}_Single_AD_Admin_List.json
    $ADAdmin=az postgres server ad-admin list -g $ser_rg -s $ser_dom | ConvertFrom-Json| ForEach-Object { $_ | Add-Member -MemberType NoteProperty -Name 'Servername' -Value $ServerName -PassThru} | ConvertTo-Json
    $ADAdmin = $ADAdmin|ConvertFrom-Json | ConvertTo-HashTable
    if($ADAdmin -is [Hashtable])
    {
        CMF-Read-Hashtable $ADAdmin
    }else
    {
	    CMF-Read-Array $ADAdmin
    }
     
    $Admin+=$global:alldata
    $global:alldata=@()
    
    #Firewall Rule
    
    #$FW_list = az postgres server firewall-rule list -g $ser_rg -s $ser_dom | Out-File $folder\Output\Single\$ServerName\${ServerName}_Single_FW_Rule.json
    $FWList=az postgres server firewall-rule list -g $ser_rg -s $ser_dom| ConvertFrom-Json| ForEach-Object { $_ | Add-Member -MemberType NoteProperty -Name 'Servername' -Value $ServerName -PassThru} | ConvertTo-Json
    $FWList = $FWList|ConvertFrom-Json | ConvertTo-HashTable
    if($FWList -is [Hashtable])
    {
        CMF-Read-Hashtable $FWList
    }else
    {
	    CMF-Read-Array $FWList
    }
    $Firewall+=$global:alldata
    $global:alldata=@()

    #Replica List
    
    $Replica_List=az postgres server replica list -g $ser_rg -s $ser_dom|ConvertFrom-Json| ConvertTo-HashTable
    if($Replica_List -is [Hashtable])
    {
        CMF-Read-Hashtable $Replica_List
    }else
    {
	    CMF-Read-Array $Replica_List
    }
    $Replica+=$global:alldata
    $global:alldata=@()

    #CMK List

    $KeyList=az postgres server key list -g $ser_rg -s $ser_dom| ConvertFrom-Json| ForEach-Object { $_ | Add-Member -MemberType NoteProperty -Name 'Servername' -Value $ServerName -PassThru} | ConvertTo-Json
    $KeyList = $KeyList|ConvertFrom-Json | ConvertTo-HashTable
    if($KeyList -is [Hashtable])
    {
        CMF-Read-Hashtable $KeyList
    }else
    {
            CMF-Read-Array $KeyList
    }
    $CMKey+=$global:alldata
    $global:alldata=@()

    #MSDef List

    $MSDefList=az security pricing show -n 'OpenSourceRelationalDatabases'| ConvertFrom-Json| ForEach-Object { $_ | Add-Member -MemberType NoteProperty -Name 'Servername' -Value $ServerName -PassThru} | ConvertTo-Json
    $MSDefList = $MSDefList|ConvertFrom-Json | ConvertTo-HashTable
    if($MSDefList -is [Hashtable])
    {
        CMF-Read-Hashtable $MSDefList
    }else
    {
            CMF-Read-Array $MSDefList
    }
    $MSDef+=$global:alldata
    $global:alldata=@()

    #Serend List

    $SerEndList=az postgres server vnet-rule list --resource-group $ser_rg  --server-name $ServerName | ConvertFrom-Json| ForEach-Object { $_ | Add-Member -MemberType NoteProperty -Name 'Servername' -Value $ServerName -PassThru} | ConvertTo-Json
    $SerEndList = $SerEndList|ConvertFrom-Json | ConvertTo-HashTable
    if($SerEndList -is [Hashtable])
    {
        CMF-Read-Hashtable $SerEndList
    }else
    {
            CMF-Read-Array $SerEndList
    }
    $SerEnd+=$global:alldata
    $global:alldata=@()

    #Pvtend List

    $PvtEndList=az network private-endpoint-connection list --type Microsoft.DBforPostgreSQL/servers -g $ser_rg -n $ser_dom | ConvertFrom-Json| ForEach-Object { $_ | Add-Member -MemberType NoteProperty -Name 'Servername' -Value $ServerName -PassThru} | ConvertTo-Json
    $PvtEndList = $PvtEndList|ConvertFrom-Json | ConvertTo-HashTable
    if($PvtEndList -is [Hashtable])
    {
        CMF-Read-Hashtable $PvtEndList
    }else
    {
            CMF-Read-Array $PvtEndList
    }
    $PvtEnd+=$global:alldata
    $global:alldata=@()
    
  }
  catch
  {
      throw $_.Exception.Message  
      throw $_.Exception.Response.StatusCode.Value__
      exitcode
    }
    }
      else
      {
      Write-Host "PostgreSQL Single server [$ServerName] is in ""$userVisibleState"" state, Database(s) information can't be captured for [$ServerName]." -ForegroundColor Red
      }
      }

    $Serverdata| ForEach-Object {[PSCustomObject]$_}  | Export-CSV -NoTypeInformation -Path $PSScriptRoot\Output\Server_List.csv -Append -Force
    $DBList| ForEach-Object {[PSCustomObject]$_} | Export-CSV -NoTypeInformation -Path $PSScriptRoot\Output\DB_Details.csv -Append -Force
    $ServerConf| ForEach-Object {[PSCustomObject]$_} | Export-CSV -NoTypeInformation -Path $PSScriptRoot\Output\Configuration_Data.csv -Append -Force
    $Admin| ForEach-Object {[PSCustomObject]$_} | Export-CSV -NoTypeInformation -Path $PSScriptRoot\Output\AD_Admin.csv -Append -Force
    $Firewall| ForEach-Object {[PSCustomObject]$_} | Export-CSV -NoTypeInformation -Path $PSScriptRoot\Output\FW_List.csv -Append -Force
    $Replica| ForEach-Object {[PSCustomObject]$_} | Export-CSV -NoTypeInformation -Path $PSScriptRoot\Output\Replica_List.csv -Append -Force
    $CMKey| ForEach-Object {[PSCustomObject]$_} | Export-CSV -NoTypeInformation -Path $PSScriptRoot\Output\CMK_List.csv -Append -Force
    $MSDef| ForEach-Object {[PSCustomObject]$_} | Export-CSV -NoTypeInformation -Path $PSScriptRoot\Output\MSDef_List.csv -Append -Force
    $SerEnd| ForEach-Object {[PSCustomObject]$_} | Export-CSV -NoTypeInformation -Path $PSScriptRoot\Output\SerEnd_List.csv -Append -Force
    $PvtEnd| ForEach-Object {[PSCustomObject]$_} | Export-CSV -NoTypeInformation -Path $PSScriptRoot\Output\PvtEnd_List.csv -Append -Force
    
     if($Outfiledata -ne $null){
    $Outfiledata | select Host_Name,Resource_Group,Port,VCore,Auth_Type,User_ID,Password,DB_Name,tenant,Subscription_ID,Approval_Status,SSL_Mode | Export-Csv -NoTypeInformation $PSScriptRoot\CMF_PostgreSQL_Server_Input_file.csv
    }
    }
    Write-Host "=============================================================================================================="  
    Write-Host "Information gathered ==> $folder\Output\ " -ForegroundColor Green -BackgroundColor Black
    Write-Host "PostgreSQL Server List created ==> $PSScriptRoot\CMF_PostgreSQL_Server_Input_file.csv " -ForegroundColor Green -BackgroundColor Black
    Write-Host "Update mandatory fields along with ""Approval_Status"" column in above CSV and execute script ""CMF-PostgreSQL-Windows.ps1"" to gather database level info." -ForegroundColor Yellow -BackgroundColor Black
    Write-Host "==============================================================================================================" 
  
   
$filenames = 'AD_Admin.csv', 'Configuration_Data.csv', 'DB_Details.csv', 'FW_List.csv','Replica_List.csv','Server_List.csv','Single_Server_List.json'
foreach ($filename in $filenames) {
$found=$false; 
Get-ChildItem -Path $PSScriptRoot\Output\ -Recurse | ForEach-Object {if($filename -eq $_.Name) {$found=$true;CONTINUE}$found=$false;}-End { if($found -ne $true) { Write-Host 'FILE ' $filename ' missing in the folder.' -foregroundcolor red}
 }
 }
 If(-Not(Test-Path $PSScriptRoot\CMF_PostgreSQL_Server_Input_file.csv))
 {
 Write-Host 'FILE ' CMF_PostgreSQL_Server_Input_file.csv ' missing in the folder.' -foregroundcolor red
 }

exitcode

